@@ -106,6 +106,7 @@ static void     gdm_manager_finalize    (GObject         *object);
 static void create_seed_session_for_display (GdmManager *manager,
                                              GdmDisplay *display,
                                              uid_t       allowed_user);
+static void     touch_ran_once_marker_file  (GdmManager *manager);
 
 static gpointer manager_object = NULL;
 
@@ -1203,6 +1204,7 @@ set_up_greeter_session (GdmManager *manager,
         g_free (allowed_user);
 
         gdm_display_start_greeter_session (display);
+        touch_ran_once_marker_file (manager);
 }
 
 static void
@@ -1608,7 +1610,6 @@ on_session_client_connected (GdmSession      *session,
                              GdmManager      *manager)
 {
         GdmDisplay *display;
-        gboolean timed_login_enabled;
         char    *username;
         int      delay;
         gboolean enabled, display_is_local;
@@ -1623,10 +1624,10 @@ on_session_client_connected (GdmSession      *session,
 
         g_object_get (display, "is-local", &display_is_local, NULL);
 
-        timed_login_enabled = FALSE;
+        enabled = FALSE;
         gdm_display_get_timed_login_details (display, &enabled, &username, &delay, NULL);
 
-        if (! timed_login_enabled) {
+        if (! enabled) {
                 return;
         }
 
@@ -1859,7 +1860,7 @@ start_autologin_conversation_if_necessary (GdmManager *manager,
 }
 
 static void
-touch_marker_file (GdmManager *manager)
+touch_ran_once_marker_file (GdmManager *manager)
 {
         int fd;
 
@@ -1965,7 +1966,6 @@ create_seed_session_for_display (GdmManager *manager,
         g_object_set_data_full (G_OBJECT (display), "gdm-seed-session", g_object_ref (session), (GDestroyNotify) g_object_unref);
 
         start_autologin_conversation_if_necessary (manager, display, session);
-        touch_marker_file (manager);
 }
 
 static void
